@@ -20,9 +20,9 @@
 # This metadata block gets parsed by setup.py - use single quotes only
 __tagline__   = 'Media aggregator and podcast client'
 __author__    = 'Thomas Perl <thp@gpodder.org>'
-__version__   = '3.9.0'
-__date__      = '2016-02-03'
-__relname__   = 'Helium Levitator'
+__version__   = '3.9.2'
+__date__      = '2016-11-30'
+__relname__   = 'Canyonero'
 __copyright__ = '© 2005-2016 Thomas Perl and the gPodder Team'
 __license__   = 'GNU General Public License, version 3 or later'
 __url__       = 'http://gpodder.org/'
@@ -37,12 +37,12 @@ import locale
 
 # Check if real hard dependencies are available
 try:
-    import feedparser
+    import podcastparser
 except ImportError:
     print """
-  Error: Module "feedparser" (python-feedparser) not found.
-         The feedparser module can be downloaded from
-         http://code.google.com/p/feedparser/
+  Error: Module "podcastparser" (python-podcastparser) not found.
+         The podcastparser module can be downloaded from
+         http://gpodder.org/podcastparser/
 
   From a source checkout, you can download local copies of all
   CLI dependencies for debugging (will be placed into "src/"):
@@ -50,7 +50,7 @@ except ImportError:
       python tools/localdepends.py
 """
     sys.exit(1)
-del feedparser
+del podcastparser
 
 try:
     import mygpoclient
@@ -58,7 +58,7 @@ except ImportError:
     print """
   Error: Module "mygpoclient" (python-mygpoclient) not found.
          The mygpoclient module can be downloaded from
-         http://thp.io/2010/mygpoclient/
+         http://gpodder.org/mygpoclient/
 
   From a source checkout, you can download local copies of all
   CLI dependencies for debugging (will be placed into "src/"):
@@ -81,7 +81,7 @@ del sqlite3
 
 
 # The User-Agent string for downloads
-user_agent = 'gPodder/%s (+%s)' % (__version__, __url__)
+user_agent = 'gPodder/%s (+%s) %s/%s' % (__version__, __url__, platform.system(), platform.release())
 
 # Are we running in GUI or console mode?
 class UI(object):
