@@ -44,7 +44,7 @@ import time
 import datetime
 
 import hashlib
-import feedparser
+import podcastparser
 import collections
 import string
 
@@ -60,12 +60,9 @@ class gPodderFetcher(feedcore.Fetcher):
     """
     custom_handlers = []
 
-    def __init__(self):
-        feedcore.Fetcher.__init__(self, gpodder.user_agent)
-
     def fetch_channel(self, channel):
         etag = channel.http_etag
-        modified = feedparser._parse_date(channel.http_last_modified)
+        modified = podcastparser.parse_pubdate(channel.http_last_modified)
         # If we have a username or password, rebuild the url with them included
         # Note: using a HTTPBasicAuthHandler would be pain because we need to
         # know the realm. It can be done, but I think this method works, too
@@ -144,145 +141,54 @@ class PodcastEpisode(PodcastModelObject):
                 youtube.is_video_link(self.link))
 
     @classmethod
-    def from_feedparser_entry(cls, entry, channel):
+    def from_podcastparser_entry(cls, entry, channel):
         episode = cls(channel)
-        episode.guid = entry.get('id', '')
+        episode.guid = entry['guid']
+        episode.title = entry['title']
+        episode.link = entry['link']
+        episode.description = entry['description']
+        episode.total_time = entry['total_time']
+        episode.published = entry['published']
+        episode.payment_url = entry['payment_url']
 
-        # Replace multi-space and newlines with single space (Maemo bug 11173)
-        episode.title = re.sub('\s+', ' ', entry.get('title', ''))
-        episode.link = entry.get('link', '')
-        if 'content' in entry and len(entry['content']) and \
-                entry['content'][0].get('type', '') == 'text/html':
-            episode.description = entry['content'][0].value
-        else:
-            episode.description = entry.get('summary', '')
+        audio_available = any(enclosure['mime_type'].startswith('audio/') for enclosure in entry['enclosures'])
+        video_available = any(enclosure['mime_type'].startswith('video/') for enclosure in entry['enclosures'])
 
-        # Fallback to subtitle if summary is not available
-        if not episode.description:
-            episode.description = entry.get('subtitle', '')
-
-        try:
-            total_time = 0
-
-            # Parse iTunes-specific podcast duration metadata
-            itunes_duration = entry.get('itunes_duration', '')
-            if itunes_duration:
-                total_time = util.parse_time(itunes_duration)
-
-            # Parse time from YouTube descriptions if it's a YouTube feed
-            if youtube.is_youtube_guid(episode.guid):
-                result = re.search(r'Time:<[^>]*>\n<[^>]*>([:0-9]*)<',
-                        episode.description)
-                if result:
-                    youtube_duration = result.group(1)
-                    total_time = util.parse_time(youtube_duration)
-
-            episode.total_time = total_time
-        except:
-            pass
-
-        episode.published = feedcore.get_pubdate(entry)
-
-        enclosures = entry.get('enclosures', [])
-        media_rss_content = entry.get('media_content', [])
-        audio_available = any(e.get('type', '').startswith('audio/') \
-                for e in enclosures + media_rss_content)
-        video_available = any(e.get('type', '').startswith('video/') \
-                for e in enclosures + media_rss_content)
-
-        # XXX: Make it possible for hooks/extensions to override this by
-        # giving them a list of enclosures and the "self" object (podcast)
-        # and letting them sort and/or filter the list of enclosures to
-        # get the desired enclosure picked by the algorithm below.
-        filter_and_sort_enclosures = lambda x: x
-
-        payment_info = [link['href'] for link in entry.get('links', [])
-            if link['rel'] == 'payment']
-        if payment_info:
-            episode.payment_url = payment_info[0]
-
-        # Enclosures
-        for e in filter_and_sort_enclosures(enclosures):
-            episode.mime_type = e.get('type', 'application/octet-stream')
-            if episode.mime_type == '':
-                # See Maemo bug 10036
-                logger.warn('Fixing empty mimetype in ugly feed')
-                episode.mime_type = 'application/octet-stream'
-
-            if '/' not in episode.mime_type:
-                continue
+        for enclosure in entry['enclosures']:
+            episode.mime_type = enclosure['mime_type']
 
             # Skip images in feeds if audio or video is available (bug 979)
             # This must (and does) also look in Media RSS enclosures (bug 1430)
-            if episode.mime_type.startswith('image/') and \
-                    (audio_available or video_available):
+            if episode.mime_type.startswith('image/') and (audio_available or video_available):
                 continue
 
             # If we have audio or video available later on, skip
             # 'application/octet-stream' data types (fixes Linux Outlaws)
-            if episode.mime_type == 'application/octet-stream' and \
-                    (audio_available or video_available):
+            if episode.mime_type == 'application/octet-stream' and (audio_available or video_available):
                 continue
 
-            episode.url = util.normalize_feed_url(e.get('href', ''))
+            episode.url = util.normalize_feed_url(enclosure['url'])
             if not episode.url:
                 continue
 
-            try:
-                episode.file_size = int(e.length) or -1
-            except:
-                episode.file_size = -1
-
+            episode.file_size = enclosure['file_size']
             return episode
 
-        # Media RSS content
-        for m in filter_and_sort_enclosures(media_rss_content):
-            episode.mime_type = m.get('type', 'application/octet-stream')
-            if '/' not in episode.mime_type:
-                continue
-
-            # Skip images in Media RSS if we have audio/video (bug 1444)
-            if episode.mime_type.startswith('image/') and \
-                    (audio_available or video_available):
-                continue
-
-            episode.url = util.normalize_feed_url(m.get('url', ''))
-            if not episode.url:
-                continue
-
-            try:
-                episode.file_size = int(m.get('filesize', 0)) or -1
-            except:
-                episode.file_size = -1
-
-            try:
-                episode.total_time = int(m.get('duration', 0)) or 0
-            except:
-                episode.total_time = 0
+        # Brute-force detection of the episode link
+        episode.url = util.normalize_feed_url(entry['link'])
+        if not episode.url:
+            return None
 
+        if any(mod.is_video_link(episode.url) for mod in (youtube, vimeo, escapist_videos)):
             return episode
 
-        # Brute-force detection of any links
-        for l in entry.get('links', ()):
-            episode.url = util.normalize_feed_url(l.get('href', ''))
-            if not episode.url:
-                continue
+        # Check if we can resolve this link to a audio/video file
+        filename, extension = util.filename_from_url(episode.url)
+        file_type = util.file_type_by_extension(extension)
 
-            if ( youtube.is_video_link(episode.url) or \
-                    vimeo.is_video_link(episode.url) or \
-                    escapist_videos.is_video_link(episode.url) ):
-                return episode
-
-            # Check if we can resolve this link to a audio/video file
-            filename, extension = util.filename_from_url(episode.url)
-            file_type = util.file_type_by_extension(extension)
-            if file_type is None and hasattr(l, 'type'):
-                extension = util.extension_from_mimetype(l.type)
-                file_type = util.file_type_by_extension(extension)
-
-            # The link points to a audio or video file - use it!
-            if file_type is not None:
-                return episode
+        # The link points to a audio or video file - use it!
+        if file_type is not None:
+            return episode
 
         return None
 
@@ -775,6 +681,7 @@ class PodcastChannel(PodcastModelObject):
         self.download_folder = None
         self.pause_subscription = False
         self.sync_to_mp3_player = True
+        self.cover_thumb = None
 
         self.section = _('Other')
         self._common_prefix = None
@@ -1009,29 +916,11 @@ class PodcastChannel(PodcastModelObject):
         self.remove_unreachable_episodes(existing, seen_guids, max_episodes)
 
     def _consume_updated_feed(self, feed, max_episodes=0):
-        # Cover art URL
-        cover_url = None
-        if hasattr(feed.feed, 'image'):
-            for attribute in ('href', 'url'):
-                new_value = getattr(feed.feed.image, attribute, None)
-                if new_value is not None:
-                    cover_url = new_value
-        elif hasattr(feed.feed, 'icon'):
-            cover_url = feed.feed.icon
-
-        # Payment URL information
-        payment_info = [link['href'] for link in feed.feed.get('links', [])
-            if link['rel'] == 'payment']
-        if payment_info:
-            payment_url = payment_info[0]
-        else:
-            payment_url = None
-
-        self._consume_metadata(feed.feed.get('title', self.url),
-                feed.feed.get('link', self.link),
-                feed.feed.get('subtitle', self.description),
-                cover_url,
-                payment_url)
+        self._consume_metadata(feed.get('title', self.url),
+                               feed.get('link', self.link),
+                               feed.get('description', ''),
+                               feed.get('cover_url', None),
+                               feed.get('payment_url', None))
 
         # Load all episodes to update them properly.
         existing = self.get_all_episodes()
@@ -1040,7 +929,7 @@ class PodcastChannel(PodcastModelObject):
         # because if the feed lists items in ascending order and has >
         # max_episodes old episodes, new episodes will not be shown.
         # See also: gPodder Bug 1186
-        entries = sorted(feed.entries, key=feedcore.get_pubdate, reverse=True)
+        entries = sorted(feed.get('episodes', []), key=lambda episode: episode['published'], reverse=True)
 
         # We can limit the maximum number of entries that gPodder will parse
         if max_episodes > 0 and len(entries) > max_episodes:
@@ -1060,18 +949,8 @@ class PodcastChannel(PodcastModelObject):
 
         # Search all entries for new episodes
         for entry in entries:
-            episode = self.EpisodeClass.from_feedparser_entry(entry, self)
+            episode = self.EpisodeClass.from_podcastparser_entry(entry, self)
             if episode is not None:
-                if not episode.title:
-                    logger.warn('Using filename as title for %s', episode.url)
-                    basename = os.path.basename(episode.url)
-                    episode.title, ext = os.path.splitext(basename)
-
-                # Maemo bug 12073
-                if not episode.guid:
-                    logger.warn('Using download URL as GUID for %s', episode.title)
-                    episode.guid = episode.url
-
                 seen_guids.add(episode.guid)
             else:
                 continue
@@ -1140,12 +1019,14 @@ class PodcastChannel(PodcastModelObject):
             elif result.status == feedcore.UPDATED_FEED:
                 self._consume_updated_feed(result.feed, max_episodes)
             elif result.status == feedcore.NEW_LOCATION:
-                url = result.feed.href
+                url = result.feed
                 logger.info('New feed location: %s => %s', self.url, url)
                 if url in set(x.url for x in self.model.get_podcasts()):
                     raise Exception('Already subscribed to ' + url)
                 self.url = url
-                self._consume_updated_feed(result.feed, max_episodes)
+                # With the updated URL, fetch the feed again
+                self.update(max_episodes)
+                return
             elif result.status == feedcore.NOT_MODIFIED:
                 pass
 
